% Chain rection:
%
%   A --> B -- > C
%   
%   where[A0] > 0, [B0] > 0, and [C0] = 0
%
clc
close all
clear
global tData
global cData % a matrix
global incr
fprintf('Reaction: A --> B -- > CD\n');
fprintf('where[A0] > 0, [B0] > 0, and [C0] = 0\n');
fprintf('All reactions are first order\n');
fprintf('Please wait ...\n');
A0 = 100;
B0 = 10;
k1 = 0.5;
k2 = 0.1;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[ConcObs,ConcTh,tArr] = react(A0,B0,k1,k2,tArr,0);
tData = tArr;
cData = ConcObs;

fcn = @rkf5;
nvars = 4;
lb = [A0 B0 k1 k2]/1.05;
ub = 1.05^2*lb;
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000, ...
   'DisplayInterval', 100);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('\nBest A0=%f, B0=%f, k1=%f k2=%f\n', bestX(1), bestX(2), bestX(3), bestX(4));
fprintf('Best fx = %e\n', bestFx);
% [bestX, bestFx] = scout([50 .01], [200 .1], [10 .01], [.1 .001], 10000, 100, 50, false, true)
n = length(tArr);
ConcCalc = zeros(n,3);
A00 = bestX(1);
B00 = bestX(2);
kk1 = bestX(3);
kk2 = bestX(4);
rB = kk1/(kk2-kk1);
for i=1:n
  t = tArr(i);
  ConcCalc(i,1) = A00*exp(-kk1*t);
  ConcCalc(i,2) = A00*rB*(exp(-kk1*t) - exp(-kk2*t)) + B00*exp(-kk2*t);
  ConcCalc(i,3) = A00  + B00 - ConcCalc(i,1) - ConcCalc(i,2);
end  

for j=1:3
  figure(j)
  plot(tArr,ConcTh(:,j),tArr,ConcObs(:,j),'r',tArr,ConcCalc(:,j),'g'); 
  if j == 1
    title('Concentration of A');
  elseif j == 2
    title('Concentration of B');
  else
    title('Concentration of C');
  end
  xlabel('Time');
  ylabel('Concentration');
  grid;
end

figure(4)
plot(tArr,ConcTh(:,1),tArr,ConcTh(:,2),'r',tArr,ConcTh(:,3),'g'); 
title('Theoretical concentrations of A, B, and C');
xlabel('Time');
ylabel('Concentration');
grid;

figure(5)
plot(tArr,ConcObs(:,1),tArr,ConcObs(:,2),'r',tArr,ConcObs(:,3),'g'); 
title('Observed concentrations of A, B, and C');
xlabel('Time');
ylabel('Concentration');
grid;

figure(6)
plot(tArr,ConcCalc(:,1),tArr,ConcCalc(:,2),'r',tArr,ConcCalc(:,3),'g'); 
title('Calculated concentrations of A, B, and C');
xlabel('Time');
ylabel('Concentration');
grid;

for j =1:3
  if j == 1
    fprintf('For concentration of A\n');
  elseif j == 2
    fprintf('For concentration of B\n');
  else
    fprintf('For concentration of C\n');
  end
  for i=1:n
    if ConcTh(i,j)~=0
      err1 = (ConcCalc(i,j) - ConcTh(i,j))/ConcTh(i,j)*100;
      err2 = (ConcCalc(i,j) - ConcObs(i,j))/ConcObs(i,j)*100;
      fprintf('%f %f %f %f %f\n', ConcTh(i,j), ConcObs(i,j), ConcCalc(i,j), err1, err2);
    end
  end
  fprintf('\n');
  fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(ConcTh(:,j),ConcCalc(:,j)));
  fprintf('R^2 = %f for comparing observed and calculated values\n\n', rsqr(ConcObs(:,j),ConcCalc(:,j)));
end

