function sumSqrErr = rkf5(x)
% rkf5 implements Runge-Kutta-Fehlberg
% A --> B --> C --> D where B0 = C0 = D0 = 0
  global tData
  global cData  
  global incr;
  A0 = x(1);
  kr1 = x(2);
  kr2 = x(3);
  kr3 = x(4);
  iData = 1;  
  nData = length(tData);
  h = incr/10;
  nSteps = fix((tData(nData)-tData(1))/h + 0.5);
  sumSqrErr = 0;
  t = tData(1);
  y = [A0 0 0];
  for iter=1:nSteps
    if t+h > tData(iData)
      for j=1:3   
        sumSqrErr = sumSqrErr + (y(j) - cData(iData,j))^2;
      end
      iData = iData + 1;
    end
    k1 = h*fx(y,kr1,kr2,kr3);
    k2 = h*fx(y+k1/4,kr1,kr2,kr3);
    k3 = h*fx(y+(3*k1+9*k2)/32,kr1,kr2,kr3);
    k4 = h*fx(y+(1932*k1-7200*k2+7296*k3)/2197,kr1,kr2,kr3);
    k5 = h*fx(y+439/216*k1-8*k2+3680/513*k3-845/4104*k4,kr1,kr2,kr3);
    k6 = h*fx(y - 8/27*k1 + 2*k2 -3544/2565*k3 +1859/4104*k4 -11/40*k5,kr1,kr2,kr3);
    y = y + 16/135*k1 + 6656/12825*k3 + 28561/56430*k4 - 9/50*k5 + 2/55*k6;
    t = t + h;
  end
  sumSqrErr = sumSqrErr / 3;
end

function y = fx(x,k1,k2,k3)
  y(1) = -k1*x(1);
  y(2) = k1*x(1)-k2*x(2);
  y(3) = k2*x(2)-k3*x(3);
end


