% Reversible Reaction
% A <==> B
% A0 > 0 and B0 > 0
% All reactions are first order
clc
close
clear all
global tData
global aData
global bData
global incr
fprintf('A <==> B\n');
fprintf('A0 > 0 and B0 > 0\n');
fprintf('All reactions are first order\n');
fprintf('Wait please ...\n');
A0 = 100;
B0 = 40;
k1 = 0.5;
k2 = 0.05;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[Aobs,Ath,Bobs,Bth,tArr] = react(A0,B0,k1,k2,tArr,10);
tData = tArr;
aData = Aobs;
bData = Bobs;

fcn = @rkf5;
nvars = 4;
lb = [A0 B0 k1 k2]/1.05;
ub = 1.05^2*lb;
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('\nBest A0=%f, k1=%f, B0=%f, k2=%f\n', bestX(1), bestX(3), bestX(2), bestX(4));
fprintf('Best fx = %e\n', bestFx);
n = length(tArr);
Acalc = zeros(n,1);
Bcalc = Acalc;
A00 = bestX(1);
B00 = bestX(2);
kk1 = bestX(3);
kk2 = bestX(4);
beta = kk1 + kk2;
alpha = kk1*A00 - kk2*B00;
gamma = alpha/beta;
for i=1:n
  x = gamma*(1 - exp(-beta*tArr(i)));
  Acalc(i) = A00 - x;
  Bcalc(i) = B00 + x;
end  

figure(1)
plot(tArr,Ath,tArr,Aobs,'r',tArr,Acalc,'g'); 
title('Concentration of A');
xlabel('Time');
ylabel('Concentration');
grid;

figure(2)
plot(tArr,Bth,tArr,Bobs,'r',tArr,Bcalc,'g'); 
title('Concentration of B');
xlabel('Time');
ylabel('Concentration');
grid;

fprintf('For concentration os A\n');
for i=1:n
  if Ath(i)~=0
    err1 = (Acalc(i) - Ath(i))/Ath(i)*100;
    err2 = (Acalc(i) - Aobs(i))/Aobs(i)*100;
    fprintf('%f %f %f %f %f\n', Ath(i), Aobs(i), Acalc(i), err1, err2);
  end
end
fprintf('\n\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Ath,Acalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Aobs,Acalc));

fprintf('For concentration os B\n');
for i=1:n
  if Bth(i)~=0
    err1 = (Bcalc(i) - Bth(i))/Bth(i)*100;
    err2 = (Bcalc(i) - Bobs(i))/Bobs(i)*100;
    fprintf('%f %f %f %f %f\n', Bth(i), Bobs(i), Bcalc(i), err1, err2);
  end
end
fprintf('\n\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Bth,Bcalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Bobs,Bcalc));
