% Reversible Reaction
%  A <=> B <==> C
% A0 >0, B0 = C0 = 0
% All reactions are first order
clc
close 
clear all
fprintf('Reaction: A <=> B <==> C\n');
fprintf('A0 >0, B0 = C0 = 0\n');
fprintf('All reactions are first order\n');
fprintf('Please wait ...\n\n');
global tData
global yData
global incr
A0 = 1;
k1 = 0.1;
k2 = k1/2;
k1r = k1/5;
k2r = k2/5;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10 15];
tArr = [0 3 5 7 10 13 15 17 20 22 25 30 35 42 47 50];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[ConcObs,ConcTh,tArr] = react(A0,k1,k2,k1r,k2r,tArr,10,true);
tData = tArr;
yData = ConcObs;

fcn = @rkf5;
nvars = 5;
lb = [A0 k1 k2 k1r k2r]/1.05;
ub = 1.05^2*lb;
options = optimoptions('particleswarm', 'MaxIterations', 5000, 'SwarmSize', 500);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('\nBest A0=%f, k1=%f, and k2=%f\n', bestX(1), bestX(2), bestX(3));
fprintf('k1r=%f, and k2r=%f\n', bestX(4), bestX(5));
fprintf('Best fx = %e\n', bestFx);
n = length(tArr);
ConcCalc = zeros(n,3);
A00 = bestX(1);
kk1 = bestX(2);
kk2 = bestX(3);
kk1r = bestX(4);
kk2r = bestX(5);
[~,ConcCalc,tArr] = react(A00,kk1,kk2,kk1r,kk2r,tArr,0,false);

figure(1)
plot(tArr,ConcTh(:,1),tArr,ConcTh(:,2),'r',tArr,ConcTh(:,3),'g');
title('Concencentraions of A, B, and C');
xlabel('Time');
ylabel('Concentration');
grid;

chemNames = ["A", "B", "C"];

for j=1:3
  figure(j+1)
  plot(tArr,ConcTh(:,j),tArr,ConcObs(:,j),'r',tArr,ConcCalc(:,j),'g');
  title(strcat("Concentration of ", chemNames(j)));
  xlabel('Time');
  ylabel('Concentration');
  grid;
  
  fprintf('Data for Reactant %s\n', chemNames(j));
  for i=1:n
    err1 = (ConcCalc(i,j) - ConcTh(i,j))/ConcTh(i,j)*100;
    err2 = (ConcCalc(i,j) - ConcObs(i,j))/ConcObs(i,j)*100;
    fprintf('%f %f %f %f %f\n', ConcTh(i,j), ConcObs(i,j), ConcCalc(i,j), err1, err2);
  end
  
  fprintf('\n');
  fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(ConcTh(:,j),ConcCalc(:,j)));
  fprintf('R^2 = %f for comparing observed and calculated values\n\n', rsqr(ConcObs(:,j),ConcCalc(:,j)));
end