function [ConcObs,ConcTh,tArr] = react(A0,k1,k2,k1r,k2r,tArr,percErr,bShowEqVals)
%REACT2 calculates the array of concenrations for reactant A
% Reversible Reaction
%  A <=> B <==> C
% A0 >0, B0 = C0 = 0
% All reactions are first order
  n = length(tArr);
  ConcTh = zeros(n,3);
  ConcObs = zeros(n,3);
  ConcTh(1,1) = A0;
  ConcTh(1,2) = 0;
  ConcTh(1,3) = 0;
  ConcObs(1,1) = ConcTh(1,1) * (1+percErr/100*(2*rand-1));
  ConcObs(1,2) = 0;
  ConcObs(1,3) = 0;
  a = 1;
  b = k1 + k2 + k1r + k2r;
  c = k1*k2 + k1*k2r + k1r*k2r;
  D = sqrt(b*b - 4*a*c);
  g1 = (b + D)/2/a;
  g2 = (b - D)/2/a;
  CA1 = k1r*k2r/g1/g2;
  CA2 = (g1^2 - g1*(k1r+k2r+k2) + k1r*k2r)/(g1*(g1-g2));
  CA3 = (g2^2 - g2*(k1r+k2r+k2) + k1r*k2r)/(g2*(g2-g1));
  CB0 = A0*k1;
  CB1 = k2r/g1/g2;
  CB2 = (k2r - g1)/(g1*(g1-g2));
  CB3 = (k2r - g2)/(g2*(g2-g1));
  CC0 = CB0*k2;
  CC1 = 1/(g1*g2);
  CC2 = 1/(g1*(g1-g2));
  CC3 = 1/(g2*(g2-g1));
  for i=2:n
    t = tArr(i);
    exp1 = exp(-g1*t);
    exp2 = exp(-g2*t);
    ConcTh(i,1) = A0*(CA1 + CA2*exp1 + CA3*exp2);
    ConcTh(i,2) = CB0*(CB1 + CB2*exp1 + CB3*exp2);
    ConcTh(i,3) = CC0*(CC1 + CC2*exp1 + CC3*exp2);
    for j=1:3
      ConcObs(i,j) = ConcTh(i,j) * (1+percErr/100*(2*rand-1));
    end
  end
  
  if bShowEqVals
    fprintf('Aeq = %f\n', A0*CA1);
    fprintf('Beq = %f\n', A0*k1*CB1);
    fprintf('Ceq = %f\n', A0*k1*k2*CC1);
  end
end

