function sumSqrErr = rkf5(x)
% rkf5arr implements Runge-Kutta-Fehlberg
% Chain rection:
%
%           k1     k3
%   A + B <---> Z ---> R + S
%           k2
%
%   where[A0] > 0, [B0] > 0, [Z0] = 0, [R0] = 0, and [S0] = 0
%
  global tData
  global cData  
  global incr;
  A0 = x(1);
  B0 = x(2);
  Z0 = x(3);
  R0 = x(4);
  S0 = x(5);
  kr1 = x(6);
  kr2 = x(7);
  kr3 = x(8);
  iData = 1;  
  nData = length(tData);
  h = incr/10;
  nSteps = fix((tData(nData)-tData(1))/h + 0.5);
  sumSqrErr = 0;
  t = tData(1);
  y = [A0 B0 Z0 R0 S0];
  for iter=1:nSteps
    if t+h > tData(iData)
      for j=1:5  
        sumSqrErr = sumSqrErr + (y(j) - cData(iData,j))^2;
      end
      iData = iData + 1;
    end
    k1 = h*fx(y,kr1,kr2,kr3);
    k2 = h*fx(y+k1/4,kr1,kr2,kr3);
    k3 = h*fx(y+(3*k1+9*k2)/32,kr1,kr2,kr3);
    k4 = h*fx(y+(1932*k1-7200*k2+7296*k3)/2197,kr1,kr2,kr3);
    k5 = h*fx(y+439/216*k1-8*k2+3680/513*k3-845/4104*k4,kr1,kr2,kr3);
    k6 = h*fx(y - 8/27*k1 + 2*k2 -3544/2565*k3 +1859/4104*k4 -11/40*k5,kr1,kr2,kr3);
    y = y + 16/135*k1 + 6656/12825*k3 + 28561/56430*k4 - 9/50*k5 + 2/55*k6;
    t = t + h;
  end
  sumSqrErr = sumSqrErr / 5;
end

function r = fx(Conc,k1,k2,k3)
  r = zeros(1,5);
  A = Conc(1);
  B = Conc(2);
  Z = Conc(3);
  R = Conc(4);
  S = Conc(5);
  r(1) = -k1*A*B+k2*Z;
  r(2) = r(1);
  r(3) = k1*A*B-k2*Z-k3*Z;
  r(4) = k3*Z;
  r(5) = r(4);
end


