% Chain rection:
%
%           k1     k3
%   A + B <---> Z ---> R + S
%           k2
%
%   where[A0] > 0, [B0] > 0, [Z0] = 0, [R0] = 0, and [S0] = 0
%
clc
close all
clear
global tData
global cData % a matrix
global incr
fprintf('A + B <---> Z ---> R + S\n');
fprintf(' where[A0] > 0, [B0] > 0, [Z0] = 0, [R0] = 0, and [S0] = 0\n');
fprintf('Reaction is Second-First order\n');
fprintf('Please wait ...\n');
A0 = 1;
B0 = 1;
Z0 = 0;
R0 = 0;
S0 = 0;
k1 = 0.5;
k2 = 0.1;
k3 = 0.2;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[ConcObs,ConcTh,tArr] = react(A0,B0,Z0,R0,S0,k1,k2,k3,tArr,0);
tData = tArr;
cData = ConcObs;

fcn = @rkf5;
nvars = 8;
c = 1.05;
lb = [A0 B0 Z0 R0 S0 k1 k2 k3]/c;
ub = [c*A0 c*B0 .1 .1 .1 c*k1 c*k2 c*k3];
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000, ...
   'DisplayInterval', 100);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('\nBest initial concetration:\n'); disp(bestX(1:5));
fprintf('\nBest reaction rate constants:\n'); disp(bestX(6:8));

fprintf('Best fx = %e\n', bestFx);
n = length(tArr);
ConcCalc = zeros(n,4);
A00 = bestX(1);
B00 = bestX(2);
Z00 = bestX(3);
R00 = bestX(4);
S00 = bestX(5);
kk1 = bestX(6);
kk2 = bestX(7);
kk3 = bestX(8);


[~,ConcCalc,tArr] = react(A00,B00,Z00,R00,S00,kk1,kk2,kk3,tArr,0);

for j=1:5
  figure(j)
  plot(tArr,ConcTh(:,j),tArr,ConcObs(:,j),'r',tArr,ConcCalc(:,j),'g'); 
  if j == 1
    title('Concentration of A');
  elseif j == 2
    title('Concentration of B');
  elseif j == 3
    title('Concentration of Z');
  elseif j == 4
    title('Concentration of R');
  else
    title('Concentration of S');
  end
  xlabel('Time');
  ylabel('Concentration');
  grid;
end

figure(5)
plot(tArr,ConcTh(:,1),tArr,ConcTh(:,2),'r',tArr,ConcTh(:,3),'g',tArr,ConcTh(:,4),'y',tArr,ConcTh(:,5),'r'); 
title('Theoretical concentrations of A, B, Z, R and S');
xlabel('Time');
ylabel('Concentration');
grid;

figure(6)
plot(tArr,ConcObs(:,1),tArr,ConcObs(:,2),'r',tArr,ConcObs(:,3),'g',tArr,ConcObs(:,4),'y',tArr,ConcObs(:,5),'r'); 
title('Observed concentrations of A, B, Z, R and S');
xlabel('Time');
ylabel('Concentration');
grid;

figure(7)
plot(tArr,ConcCalc(:,1),tArr,ConcCalc(:,2),'r',tArr,ConcCalc(:,3),'g',tArr,ConcCalc(:,4),'y',tArr,ConcCalc(:,5),'r'); 
title('Calculated concentrations of A, B, Z, R and S');
xlabel('Time');
ylabel('Concentration');
grid;


for j =1:5
  if j == 1
    fprintf('For concentration of A\n');
  elseif j == 2
    fprintf('For concentration of B\n');
  elseif j == 3
    fprintf('For concentration of Z\n');
  elseif j == 4
    fprintf('For concentration of R\n');
  else
    fprintf('For concentration of S\n');    
  end
  for i=1:n
    if ConcTh(i,j)~=0
      err1 = (ConcCalc(i,j) - ConcTh(i,j))/ConcTh(i,j)*100;
      err2 = (ConcCalc(i,j) - ConcObs(i,j))/ConcObs(i,j)*100;
      fprintf('%f %f %f %f %f\n', ConcTh(i,j), ConcObs(i,j), ConcCalc(i,j), err1, err2);
    end
  end
  fprintf('\n');
  fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(ConcTh(:,j),ConcCalc(:,j)));
  fprintf('R^2 = %f for comparing observed and calculated values\n\n', rsqr(ConcObs(:,j),ConcCalc(:,j)));
end

