% Reversible Reaction:
% A + B <---> C
% A0, B0 > 0, A0<>B0. and C0 = 0
% -rA = k1*A*B - k2*C
clc
close 
clear all
fprintf('A + B <---> C\n');
fprintf('A0 = B0 > 0 and C0 = 0\n');
fprintf('-rA = k1*A*B - k2*C\n');
fprintf('Wait please ...\n\n');
global tData
global yData
global incr
A0 = 1;
B0 = 0.95;
k1 = 0.5;
k2 = 0.1;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[Aobs,Ath,Bobs,Bth,tArr] = react(A0,B0,k1,k2,tArr,10);
Cth = A0 - Ath;
Cobs = A0 - Aobs;
tData = tArr;
yData = Aobs;

fcn = @rkf5;
nvars = 4;
cf = 1.05;
lb = [A0 B0 k1 k2]/cf;
ub = cf^2*lb;
options = optimoptions('particleswarm', 'Display', 'iter');
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub);
fprintf('\nBest A0=%f, B0=%f, k1=%f, and k2=%f\n', bestX(1), bestX(2), bestX(3), bestX(4));
fprintf('Best fx = %e\n', bestFx);
n = length(tArr);
Acalc = zeros(n,1);
A00 = bestX(1);
B00 = bestX(2);
kk1 = bestX(3);
kk2 = bestX(4);
[~,Acalc,~,Bcalc,tArr] = react(A00,B00,kk1,kk2,tArr,0);
Ccalc = A00 - Acalc;
figure(1)
plot(tArr,Ath,tArr,Aobs,'r',tArr,Acalc,'g'); 
title('Concentration of A');
xlabel('Time');
ylabel('Concentration');
grid;

figure(2)
plot(tArr,Bth,tArr,Bobs,'r',tArr,Bcalc,'g'); 
title('Concentration of B');
xlabel('Time');
ylabel('Concentration');
grid;

figure(3)
plot(tArr,Cth,tArr,Cobs,'r',tArr,Ccalc,'g'); 
title('Concentration of C');
xlabel('Time');
ylabel('Concentration');
grid;

fprintf('For reactant A:\n');
for i=1:n
  err1 = (Acalc(i) - Ath(i))/Ath(i)*100;
  err2 = (Acalc(i) - Aobs(i))/Aobs(i)*100;
  fprintf('%f %f %f %f %f\n', Ath(i), Aobs(i), Acalc(i), err1, err2);
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Ath,Acalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Aobs,Acalc));

fprintf('\nFor reactant B:\n');
for i=1:n
  err1 = (Bcalc(i) - Bth(i))/Bth(i)*100;
  err2 = (Bcalc(i) - Bobs(i))/Bobs(i)*100;
  fprintf('%f %f %f %f %f\n', Bth(i), Bobs(i), Bcalc(i), err1, err2);
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Bth,Bcalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Bobs,Bcalc));


fprintf('\nFor reactant C:\n');
for i=1:n
  err1 = (Ccalc(i) - Cth(i))/Cth(i)*100;
  err2 = (Ccalc(i) - Cobs(i))/Cobs(i)*100;
  fprintf('%f %f %f %f %f\n', Cth(i), Cobs(i), Ccalc(i), err1, err2);
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Cth,Ccalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Cobs,Ccalc));