% A + B <==> R + S, with [A0] = [B0] and [R0] = [S0] = 0
% All reactions are of second order
% source of equations is
% "Chemical Reaction Kinetics=Concepts, Methods and Case Studies"
% by Jorge Ancheyta

% Note:
% =====
% Calculations are vulnerable to simulated percent error. The lower that
% value, the better the results, especially for the calculated values for 
% the concentrations of A (and also B).
%
clc
close 
clear all
global tData
global xData
global incr
fprintf('A + B <==> R + S, with [A0] = [B0] and [R0] = [S0] = 0\n');
fprintf('All reactions are of second order\n');
fprintf('Please wait ...\n');
A0 = 1;
k1 = 0.05;
k2 = 0.05;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[Xobs,Xth,tArr] = react(A0,k1,k2,tArr,.0); %0 % error
tData = tArr;
xData = Xobs;
n = length(tArr);
Ath = A0 * (1-Xth);
Aobs = A0 * (1-Xobs);
Rth = A0 * Xth;
Robs = A0 * Xobs;
fcn = @rkf5;
nvars = 3;
lb = [.9 0.01 0.01];
ub = [1.2  0.1 .1];
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000, ...
   'DisplayInterval', 100);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('\nBest A0=%f, k1=%f, k2=%f\n', bestX(1), bestX(2), bestX(3));
fprintf('Best fx = %e\n', bestFx);
Acalc = zeros(n,1);
Rcalc = zeros(n,1);
A00 = bestX(1);
kk1 = bestX(2);
kk2 = bestX(3);
Keq = kk1/kk2;
SqrtKeq = sqrt(Keq);
xinf = SqrtKeq/(1 + SqrtKeq);
m = 2*kk1*A00*(1-xinf)/xinf;
for i=1:n
  ex =exp(-m*tArr(i));
  x =xinf*(1-ex)/(1 + (1-2*xinf)*ex);
  Acalc(i) = A00 * (1- x);
  Rcalc(i) = A00 - Acalc(i);
end  

figure(1)
plot(tArr,Ath,tArr,Aobs,'r',tArr,Acalc,'g'); 
title('Concentration of A (same as B)');
xlabel('Time');
ylabel('Concentration');
grid;

figure(2)
plot(tArr,Rth,tArr,Robs,'r',tArr,Rcalc,'g'); 
title('Concentration of R (same as S)');
xlabel('Time');
ylabel('Concentration');
grid;

fprintf('For concentration os A (and also B)\n');
for i=1:n
  if Ath(i)~=0
    err1 = (Acalc(i) - Ath(i))/Ath(i)*100;
    err2 = (Acalc(i) - Aobs(i))/Aobs(i)*100;
    fprintf('%f %f %f %f %f\n', Ath(i), Aobs(i), Acalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Ath,Acalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Aobs,Acalc));

fprintf('For concentration os R (and also S)\n');
for i=1:n
  if Rth(i)~=0
    err1 = (Rcalc(i) - Rth(i))/Rth(i)*100;
    err2 = (Rcalc(i) - Robs(i))/Robs(i)*100;
    fprintf('%f %f %f %f %f\n', Rth(i), Robs(i), Rcalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Rth,Rcalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Robs,Rcalc));
