% A <==> R, with [A0] > 0 and [R0] = 0
% (-rA) = k1*A - k2*R
% source of equations is
% "Chemical Reaction Kinetics=Concepts, Methods and Case Studies"
% by Jorge Ancheyta
clc
close 
clear all
global tData
global xData
global incr
fprintf('A <==> R at (-rA) = k1*A - k2*R\n');
fprintf('A0 > 0 and R0 = 0\n');
fprintf('Please wait ...\n');
A0 = 1;
k1 = 0.5;
k2 = 0.1;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[Xobs,Xth,xeq,tArr] = react(A0,k1,k2,tArr,0); 
tData = tArr;
xData = Xobs;
n = length(tArr);
Ath = A0 * (1-Xth);
Aobs = A0 * (1-Xobs);
Rth = A0 * Xth;
Robs = A0 * Xobs;
fcn = @rkf5;
nvars = 3;
lb = [A0 k1 k2]/1.05;
ub = 1.05^2*lb;
Vmax = (ub - lb)/10;
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000, ...
   'DisplayInterval', 100);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
fprintf('Aeq = %f and Req = %f\n', A0 - xeq, A0*xeq);
fprintf('\nBest A0=%f, k1=%f, k2=%f\n', bestX(1), bestX(2), bestX(3));
fprintf('Best fx = %e\n', bestFx);
Acalc = zeros(n,1);
Rcalc = zeros(n,1);
A00 = bestX(1);
kk1 = bestX(2);
kk2 = bestX(3);
[~,Xcalc,~,tArr] = react(A00,kk1,kk2,tArr,0);
Acalc = A0 - Xcalc;
Rcalc = A00 * Xcalc;

figure(1)
plot(tArr,Ath,tArr,Aobs,'r',tArr,Acalc,'g'); 
title('Concentration of A');
xlabel('Time');
ylabel('Concentration');
grid;

figure(2)
plot(tArr,Rth,tArr,Robs,'r',tArr,Rcalc,'g'); 
title('Concentration of R');
xlabel('Time');
ylabel('Concentration');
grid;

fprintf('For concentration os A\n');
for i=1:n
  if Ath(i)~=0
    err1 = (Acalc(i) - Ath(i))/Ath(i)*100;
    err2 = (Acalc(i) - Aobs(i))/Aobs(i)*100;
    fprintf('%f %f %f %f %f\n', Ath(i), Aobs(i), Acalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Ath,Acalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Aobs,Acalc));

fprintf('\nFor concentration os R\n');
for i=1:n
  if Rth(i)~=0
    err1 = (Rcalc(i) - Rth(i))/Rth(i)*100;
    err2 = (Rcalc(i) - Robs(i))/Robs(i)*100;
    fprintf('%f %f %f %f %f\n', Rth(i), Robs(i), Rcalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Rth,Rcalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Robs,Rcalc));
