% A <==> R, with [A0] > 0 and [R0] = 0
% (-rA) = k1*A - k2*R^2
% source of equations is
% "Chemical Reaction Kinetics=Concepts, Methods and Case Studies"
% by Jorge Ancheyta
clc
close 
clear all
global tData
global xData
global incr
fprintf('A <==> R at (-rA) = k1*A - k2*R^2\n');
fprintf('Please wait ...\n');
A0 = 1;
R0 = .0;
k1 = 0.1;
k2 = 0.05;
tArr = [0 1.2 2.2 3.3 4.5 5.3 6.1 6.9 7.2 7.9 8.1 8.8 9.3 10];
% tArr = [0 1 2 3 5 7 9 10 12];
incr = findMinDigit(tArr);
[Xobs,Xth,tArr] = react1(A0,R0,k1,k2,tArr,0); 
tData = tArr;
xData = Xobs;
n = length(tArr);
Ath = A0 * (1-Xth);
Aobs = A0 * (1-Xobs);
Rth = (A0+R0) * Xth;
Robs = (A0+R0) * Xobs;
fcn = @rkf5_1;
nvars = 4;
lb = [A0/1.05 0 k1/1.05 k2/1.05];
ub = [A0*1.05 A0 k1*1.05 k2*1.05];
Vmax = (ub - lb)/10;
options = optimoptions('particleswarm', 'SwarmSize', 500, 'MaxIterations', 5000, ...
   'DisplayInterval', 100);
[bestX, bestFx] = particleswarm(fcn,nvars,lb,ub,options);
% % % tic;
% % % [bestX,bestFx] = pso(fcn,lb,ub,Vmax,500,5000);
% % % toc
fprintf('\nBest A0=%f, R0 =%f, k1=%f, k2=%f\n', bestX(1), bestX(2), bestX(3), bestX(4));
fprintf('Best fx = %e\n', bestFx);
fprintf('Estimated xeq = %f\n', newton1(0.5,1e-6,100,k1/k2,A0,R0/A0));
Acalc = zeros(n,1);
Rcalc = zeros(n,1);
A00 = bestX(1);
R00 = bestX(2);
kk1 = bestX(3);
kk2 = bestX(4);
Keq = kk1/kk2;
Mra = R00/A00;
a = -A00 / Keq;
b = -1 -2*R00/Keq;
c = 1 - Mra^2*A00/Keq;
if b^2 > 4*a*c
  D1 = sqrt(b^2 - 4*a*c);
  K = kk1*D1;
  Bp = b+D1;
  Bm = b-D1;
  B1 = Bm/Bp; 
  Cm = Bm/2/a;
  Cp = Bp/2/a;    
  for i=1:n
    t = tArr(i);
    x = (Cp*B1 - Cm*exp(-K*t))/(exp(-K*t) - B1);
    Acalc(i) = A00 * (1- x);
    Rcalc(i) = A00 + R00 - Acalc(i);
    Acalc(i) = Acalc(i); % correction
  end
elseif b^2 < 4*a*c
  D2 = sqrt(4*a*c - b^2);
  K = kk2*D2/2;
  for i=1:n
    t = tArr(i);
    x = (D2*tan(K*t + atan(b/D2)) - b)/(2*a);
    Acalc(i) = A00 * (1- x);
    Rcalc(i) = A00 + R00 - Acalc(i);
    Acalc(i) = Acalc(i); % correction
  end    

else
 for i=1:n
    t = tArr(i);
    x = (kk1*t * b)/2 * x + (kk1*t * b)*b/4/a;
    Acalc(i) = A00 * (1- x);
    Rcalc(i) = A00 + R00 - Acalc(i);
    Acalc(i) = Acalc(i) + A0 - A00; % correction
  end       
end


figure(1)
plot(tArr,Ath,tArr,Aobs,'r',tArr,Acalc,'g'); 
title('Concentration of A');
xlabel('Time');
ylabel('Concentration');
grid;

figure(2)
plot(tArr,Rth,tArr,Robs,'r',tArr,Rcalc,'g'); 
title('Concentration of R');
xlabel('Time');
ylabel('Concentration');
grid;

fprintf('For concentration os A\n');
for i=1:n
  if Ath(i)~=0
    err1 = (Acalc(i) - Ath(i))/Ath(i)*100;
    err2 = (Acalc(i) - Aobs(i))/Aobs(i)*100;
    fprintf('%f %f %f %f %f\n', Ath(i), Aobs(i), Acalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Ath,Acalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Aobs,Acalc));

fprintf('For concentration os R\n');
for i=1:n
  if Rth(i)~=0
    err1 = (Rcalc(i) - Rth(i))/Rth(i)*100;
    err2 = (Rcalc(i) - Robs(i))/Robs(i)*100;
    fprintf('%f %f %f %f %f\n', Rth(i), Robs(i), Rcalc(i), err1, err2);
  end
end
fprintf('\n');
fprintf('R^2 = %f for comparing theoretical and calculated values\n', rsqr(Rth,Rcalc));
fprintf('R^2 = %f for comparing observed and calculated values\n', rsqr(Robs,Rcalc));
