function sumSqrErr = rkf5_3(x)
% rkf5arr implements Runge-Kutta-Fehlberg
% A <==> R, with all concentrations > 0
% (-rA) = k1*A^2 - k2*R^2
  global tData
  global xData  
  global incr;
  A0 = x(1);
  R0 = x(2);
  kr1 = x(3);
  kr2 = x(4);
  iData = 1;  
  nData = length(tData);
  h = incr/10;
  nSteps = fix((tData(nData)-tData(1))/h + 0.5);
  sumSqrErr = 0;
  t = tData(1);
  y = 0;
  Keq = kr1/kr2;
  Mra = R0/A0;
  a = A0*(1+1/Keq); 
  b = -2*A0*(1 + Mra/Keq);
  c = A0*(1 - Mra^2)/Keq;
  for iter=1:nSteps
    if t+h > tData(iData)
      sumSqrErr = sumSqrErr + (y - xData(iData))^2;
      iData = iData + 1;
    end
    k1 = h*fx(y,kr1,a,b,c);
    k2 = h*fx(y+k1/4,kr1,a,b,c);
    k3 = h*fx(y+(3*k1+9*k2)/32,kr1,a,b,c);
    k4 = h*fx(y+(1932*k1-7200*k2+7296*k3)/2197,kr1,a,b,c);
    k5 = h*fx(y+439/216*k1-8*k2+3680/513*k3-845/4104*k4,kr1,a,b,c);
    k6 = h*fx(y - 8/27*k1 + 2*k2 -3544/2565*k3 +1859/4104*k4 -11/40*k5,kr1,a,b,c);
    y = y + 16/135*k1 + 6656/12825*k3 + 28561/56430*k4 - 9/50*k5 + 2/55*k6;
    t = t + h;
  end
end

function r = fx(y,k1,a,b,c)
  r = k1*(a*y^2 + b*y + c);
end

